#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""OscCommentMacroListener.py: Hog 4 comment macro antlr4 listener for OSC."""

__author__ = "Kevin Matz"
__copyright__ = "Copyright 2018, Company 235, LLC"
__credits__ = ["Kevin Matz"]

__license__ = "MIT"
__version__ = "3.9"
__maintainer__ = "Kevin Matz"
__email__ = "kevin@company235.com"

__status__ = "Prototype"


import logging
import OscMacroDefinitions
from CommentMacroParser import CommentMacroParser
from CommentMacroListener import CommentMacroListener

logger = logging.getLogger("CommentMacro")


# https://raw.githubusercontent.com/jszheng/py3antlr4book/master/bin/pygrun
# this is a python version of TestRig
def beautify_lisp_string(in_string):
    __author__ = 'jszheng'
    indent_size = 2
    add_indent = ' ' * indent_size
    out_string = in_string[0]  # no indent for 1st (
    indent = ''
    for i in range(1, len(in_string) - 1):
        if in_string[i] == '(' and in_string[i + 1] != ' ':
            indent += add_indent
            out_string += "\n" + indent + '('
        elif in_string[i] == ')':
            out_string += ')'
            if len(indent) > 0:
                indent = indent.replace(add_indent, '', 1)
        else:
            out_string += in_string[i]
    return out_string


class OscCommentMacroListener(CommentMacroListener):
    def __init__(self, servers):
        self.osc = OscMacroDefinitions.HogDevice(servers)

    def exitMacro(self, ctx: CommentMacroParser.MacroContext):
        # print the lisp tree of this macro
        lisp_tree_str = ctx.toStringTree(recog=ctx.parser)
        logger.info(beautify_lisp_string(lisp_tree_str))

        # disallow duplicate targets
        if ctx.t is not None:
            ctx.t.targets = set(ctx.t.targets)

        # execute macro from name
        name = ctx.op.text
        try:
            self.osc.command[name](self, ctx)
        except KeyError:
            print(name + " macro is not compatable with OSC.")
        logger.debug("Exiting Macro")

    def enterTarget(self, ctx: CommentMacroParser.TargetContext):
        ctx.targets = []

    def exitTarget(self, ctx: CommentMacroParser.TargetContext):
        if isinstance(ctx.parentCtx, CommentMacroParser.TargetContext):
            ctx.parentCtx.targets.extend(ctx.targets)   # add to parent targets

    def exitNumber(self, ctx: CommentMacroParser.NumberContext):
        try:
            ctx.value = int(ctx.getText())
        except ValueError:
            ctx.value = float(ctx.getText())

        if isinstance(ctx.parentCtx, CommentMacroParser.TargetContext):
            ctx.parentCtx.targets.append(ctx.value)

    def exitSpan(self, ctx: CommentMacroParser.SpanContext):
        n1 = ctx.n1.value
        n2 = ctx.n2.value
        if (isinstance(n1, int) and isinstance(n2, int)):
            minimum = min(n1, n2)
            maximum = max(n1, n2)
            for i in (range(minimum, maximum + 1)):
                ctx.parentCtx.targets.append(i)
        else:
            logger.error("ERROR: Spans must be ranged with intigers.")
